/* Public keys for module signature verification
 *
 * Copyright (C) 2011 Red Hat, Inc. All Rights Reserved.
 * Written by David Howells (dhowells@redhat.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public Licence
 * as published by the Free Software Foundation; either version
 * 2 of the Licence, or (at your option) any later version.
 */

#include <linux/kernel.h>
#include <keys/crypto-type.h>
#include "module-verify.h"

extern __initdata const u8 modsign_public_keys[];
extern __initdata const u8 modsign_public_keys_end[];
asm(".section .init.data,\"aw\"\n"
    "modsign_public_keys:\n"
    ".incbin \"kernel.pub\"\n"
    "modsign_public_keys_end:"
    );

/*
 * We need to make sure ccache doesn't cache the .o file as it doesn't notice
 * if kernel.pub changes.
 */
static __initdata const char annoy_ccache[] = __TIME__ "foo";

/*
 * Load the compiled-in keys
 */
static __init int modsign_pubkey_init(void)
{
	pr_notice("Load module verification keys\n");

	if (preload_pgp_keys(modsign_public_keys,
			     modsign_public_keys_end - modsign_public_keys,
			     modsign_keyring, "modsign.") < 0)
		panic("Can't load module signing keys\n");

	return 0;
}
late_initcall(modsign_pubkey_init);
